# author: Kwang-Hyun Park (akaii@kw.ac.kr)

from roboid.hamster import *
import time

__hamster = Hamster()
__bpm = 60

def setName(name):
	__hamster.setName(name)

def getName():
	return __hamster.getName()

def wheels(left, right, seconds=0):
	__hamster.write(Hamster.LEFT_WHEEL, left)
	__hamster.write(Hamster.RIGHT_WHEEL, right)
	if seconds > 0:
		wait(seconds * 1000)
		__hamster.write(Hamster.LEFT_WHEEL, 0)
		__hamster.write(Hamster.RIGHT_WHEEL, 0)

def stop():
	__hamster.write(Hamster.LEFT_WHEEL, 0)
	__hamster.write(Hamster.RIGHT_WHEEL, 0)

def leds(left, right, seconds=0):
	__hamster.write(Hamster.LEFT_LED, left)
	__hamster.write(Hamster.RIGHT_LED, right)
	if seconds > 0:
		wait(seconds * 1000)
		__hamster.write(Hamster.LEFT_LED, Hamster.LED_OFF)
		__hamster.write(Hamster.RIGHT_LED, Hamster.LED_OFF)

def buzzer(hz, seconds=0):
	__hamster.write(Hamster.BUZZER, hz)
	if seconds > 0:
		wait(seconds * 1000)
		__hamster.write(Hamster.BUZZER, 0)

def tempo(bpm):
	global __bpm
	if bpm > 0:
		__bpm = bpm

def note(note, beats=0):
	global __bpm
	if note >= 0 and beats == 0:
		__hamster.write(Hamster.BUZZER, 0)
		__hamster.write(Hamster.NOTE, note)
	elif beats > 0 and __bpm > 0:
		if note == 0:
			__hamster.write(Hamster.BUZZER, 0)
			__hamster.write(Hamster.NOTE, 0)
			wait(beats * 60 * 1000 / __bpm)
		elif note > 0:
			timeout = beats * 60 * 1000 / __bpm
			tail = 0
			if timeout > 100:
				tail = 100
			__hamster.write(Hamster.BUZZER, 0)
			__hamster.write(Hamster.NOTE, note)
			wait(timeout - tail)
			__hamster.write(Hamster.NOTE, 0)
			if tail > 0:
				wait(tail)

def proximity():
	return __hamster.read(Hamster.LEFT_PROXIMITY), __hamster.read(Hamster.RIGHT_PROXIMITY)

def floor():
	return __hamster.read(Hamster.LEFT_FLOOR), __hamster.read(Hamster.RIGHT_FLOOR)

def acceleration():
	return __hamster.read(Hamster.ACCELERATION, 0), __hamster.read(Hamster.ACCELERATION, 1), __hamster.read(Hamster.ACCELERATION, 2)

def light():
	return __hamster.read(Hamster.LIGHT)

def temperature():
	return __hamster.read(Hamster.TEMPERATURE)

def reset():
	__hamster.reset()

def dispose():
	__hamster.dispose()