# author: Kwang-Hyun Park (akaii@kw.ac.kr)

import hamster_connector
from robot import Robot
from device import DeviceType
import time
import signal

SERIAL = 0
BLE = 1

class Hamster(Robot):
	ID = 'kr.robomation.physical.hamster'
	LEFT_WHEEL = 0x00400000
	RIGHT_WHEEL = 0x00400001
	BUZZER = 0x00400002
	OUTPUT_A = 0x00400003
	OUTPUT_B = 0x00400004
	TOPOLOGY = 0x00400005
	LEFT_LED = 0x00400006
	RIGHT_LED = 0x00400007
	NOTE = 0x00400008
	LINE_TRACER_MODE = 0x00400009
	LINE_TRACER_SPEED = 0x0040000a
	IO_MODE_A = 0x0040000b
	IO_MODE_B = 0x0040000c
	CONFIG_PROXIMITY = 0x0040000d
	CONFIG_GRAVITY = 0x0040000e
	CONFIG_BAND_WIDTH = 0x0040000f

	SIGNAL_STRENGTH = 0x00400010
	LEFT_PROXIMITY = 0x00400011
	RIGHT_PROXIMITY = 0x00400012
	LEFT_FLOOR = 0x00400013
	RIGHT_FLOOR = 0x00400014
	ACCELERATION = 0x00400015
	LIGHT = 0x00400016
	TEMPERATURE = 0x00400017
	INPUT_A = 0x00400018
	INPUT_B = 0x00400019
	LINE_TRACER_STATE = 0x0040001a

	TOPOLOGY_NONE = 0
	TOPOLOGY_DAISY_CHAIN = 1
	TOPOLOGY_STAR = 2
	TOPOLOGY_EXTENDED_STAR = 3

	LED_OFF = 0
	LED_BLUE = 1
	LED_GREEN = 2
	LED_CYAN = 3
	LED_RED = 4
	LED_MAGENTA = 5
	LED_YELLOW = 6
	LED_WHITE = 7

	LINE_TRACER_MODE_OFF = 0
	LINE_TRACER_MODE_BLACK_LEFT_SENSOR = 1
	LINE_TRACER_MODE_BLACK_RIGHT_SENSOR = 2
	LINE_TRACER_MODE_BLACK_BOTH_SENSORS = 3
	LINE_TRACER_MODE_BLACK_TURN_LEFT = 4
	LINE_TRACER_MODE_BLACK_TURN_RIGHT = 5
	LINE_TRACER_MODE_BLACK_MOVE_FORWARD = 6
	LINE_TRACER_MODE_BLACK_UTURN = 7
	LINE_TRACER_MODE_WHITE_LEFT_SENSOR = 8
	LINE_TRACER_MODE_WHITE_RIGHT_SENSOR = 9
	LINE_TRACER_MODE_WHITE_BOTH_SENSORS = 10
	LINE_TRACER_MODE_WHITE_TURN_LEFT = 11
	LINE_TRACER_MODE_WHITE_TURN_RIGHT = 12
	LINE_TRACER_MODE_WHITE_MOVE_FORWARD = 13
	LINE_TRACER_MODE_WHITE_UTURN = 14

	IO_MODE_ADC = 0
	IO_MODE_DI = 1
	IO_MODE_SERVO = 8
	IO_MODE_PWM = 9
	IO_MODE_DO = 10

	NOTE_OFF = 0
	NOTE_A_0 = 1
	NOTE_A_SHARP_0 = 2
	NOTE_B_FLAT_0 = 2
	NOTE_B_0 = 3
	NOTE_C_1 = 4
	NOTE_C_SHARP_1 = 5
	NOTE_D_FLAT_1 = 5
	NOTE_D_1 = 6
	NOTE_D_SHARP_1 = 7
	NOTE_E_FLAT_1 = 7
	NOTE_E_1 = 8
	NOTE_F_1 = 9
	NOTE_F_SHARP_1 = 10
	NOTE_G_FLAT_1 = 10
	NOTE_G_1 = 11
	NOTE_G_SHARP_1 = 12
	NOTE_A_FLAT_1 = 12
	NOTE_A_1 = 13
	NOTE_A_SHARP_1 = 14
	NOTE_B_FLAT_1 = 14
	NOTE_B_1 = 15
	NOTE_C_2 = 16
	NOTE_C_SHARP_2 = 17
	NOTE_D_FLAT_2 = 17
	NOTE_D_2 = 18
	NOTE_D_SHARP_2 = 19
	NOTE_E_FLAT_2 = 19
	NOTE_E_2 = 20
	NOTE_F_2 = 21
	NOTE_F_SHARP_2 = 22
	NOTE_G_FLAT_2 = 22
	NOTE_G_2 = 23
	NOTE_G_SHARP_2 = 24
	NOTE_A_FLAT_2 = 24
	NOTE_A_2 = 25
	NOTE_A_SHARP_2 = 26
	NOTE_B_FLAT_2 = 26
	NOTE_B_2 = 27
	NOTE_C_3 = 28
	NOTE_C_SHARP_3 = 29
	NOTE_D_FLAT_3 = 29
	NOTE_D_3 = 30
	NOTE_D_SHARP_3 = 31
	NOTE_E_FLAT_3 = 31
	NOTE_E_3 = 32
	NOTE_F_3 = 33
	NOTE_F_SHARP_3 = 34
	NOTE_G_FLAT_3 = 34
	NOTE_G_3 = 35
	NOTE_G_SHARP_3 = 36
	NOTE_A_FLAT_3 = 36
	NOTE_A_3 = 37
	NOTE_A_SHARP_3 = 38
	NOTE_B_FLAT_3 = 38
	NOTE_B_3 = 39
	NOTE_C_4 = 40
	NOTE_C_SHARP_4 = 41
	NOTE_D_FLAT_4 = 41
	NOTE_D_4 = 42
	NOTE_D_SHARP_4 = 43
	NOTE_E_FLAT_4 = 43
	NOTE_E_4 = 44
	NOTE_F_4 = 45
	NOTE_F_SHARP_4 = 46
	NOTE_G_FLAT_4 = 46
	NOTE_G_4 = 47
	NOTE_G_SHARP_4 = 48
	NOTE_A_FLAT_4 = 48
	NOTE_A_4 = 49
	NOTE_A_SHARP_4 = 50
	NOTE_B_FLAT_4 = 50
	NOTE_B_4 = 51
	NOTE_C_5 = 52
	NOTE_C_SHARP_5 = 53
	NOTE_D_FLAT_5 = 53
	NOTE_D_5 = 54
	NOTE_D_SHARP_5 = 55
	NOTE_E_FLAT_5 = 55
	NOTE_E_5 = 56
	NOTE_F_5 = 57
	NOTE_F_SHARP_5 = 58
	NOTE_G_FLAT_5 = 58
	NOTE_G_5 = 59
	NOTE_G_SHARP_5 = 60
	NOTE_A_FLAT_5 = 60
	NOTE_A_5 = 61
	NOTE_A_SHARP_5 = 62
	NOTE_B_FLAT_5 = 62
	NOTE_B_5 = 63
	NOTE_C_6 = 64
	NOTE_C_SHARP_6 = 65
	NOTE_D_FLAT_6 = 65
	NOTE_D_6 = 66
	NOTE_D_SHARP_6 = 67
	NOTE_E_FLAT_6 = 67
	NOTE_E_6 = 68
	NOTE_F_6 = 69
	NOTE_F_SHARP_6 = 70
	NOTE_G_FLAT_6 = 70
	NOTE_G_6 = 71
	NOTE_G_SHARP_6 = 72
	NOTE_A_FLAT_6 = 72
	NOTE_A_6 = 73
	NOTE_A_SHARP_6 = 74
	NOTE_B_FLAT_6 = 74
	NOTE_B_6 = 75
	NOTE_C_7 = 76
	NOTE_C_SHARP_7 = 77
	NOTE_D_FLAT_7 = 77
	NOTE_D_7 = 78
	NOTE_D_SHARP_7 = 79
	NOTE_E_FLAT_7 = 79
	NOTE_E_7 = 80
	NOTE_F_7 = 81
	NOTE_F_SHARP_7 = 82
	NOTE_G_FLAT_7 = 82
	NOTE_G_7 = 83
	NOTE_G_SHARP_7 = 84
	NOTE_A_FLAT_7 = 84
	NOTE_A_7 = 85
	NOTE_A_SHARP_7 = 86
	NOTE_B_FLAT_7 = 86
	NOTE_B_7 = 87
	NOTE_C_8 = 88

	__index = 0
	__hamsters = []

	def __init__(self, type=SERIAL):
		Robot.__init__(self, Hamster.ID, 'Hamster', Hamster.__index, self.__dispose)
		Hamster.__hamsters.append(self)
		Hamster.__index += 1
		self.__type = type
		self.__connector = None
		self.__init()

	def __run(self):
		while self.__running:
			if self.__decode():
				self._Robot__notifySensoryDeviceDataChanged()
				self._Robot__updateSensoryDeviceState()
				
				execute = self._Robot__execute
				if execute != None:
					execute(self)
				
				self.__encode()
				self._Robot__notifyMotoringDeviceDataChanged()
				self._Robot__updateMotoringDeviceState()
				
				time.sleep(0.01)

	def __init(self):
		import threading
		self.__createModel()
		self.__running = True
		t = threading.Thread(target=self.__run)
		self.__thread = t
		t.daemon = True
		t.start()
		if self.__type == SERIAL:
			self.__connector = hamster_connector.HamsterConnector(self.getIndex())
		if self.__connector:
			self.__connector.open()

	def __release(self):
		self.reset()
		self.__running = False
		self.__thread.join()
		if self.__connector:
			self.__connector.close()
			self.__connector = None
			print 'Hamster[%d] Disposed' % self.getIndex()

	def __dispose(self):
		self.__release()
		try:
			Hamster.__hamsters.remove(self)
		except:
			pass

	def __createModel(self):
		from hamster_roboid import HamsterRoboid
		self.__roboid = HamsterRoboid()
		self.__roboid._HamsterRoboid__createModel()
		self._Robot__addRoboid(self.__roboid)

	def __encode(self):
		if self.__connector:
			packet = self.__roboid._HamsterRoboid__encode(self.__connector.getAddress())
			self.__connector.write(packet)

	def __decode(self):
		if self.__connector:
			packet = self.__connector.read()
			if packet:
				self.__roboid._HamsterRoboid__decode(packet)
				return True
		return False

def wait(milliseconds):
	if milliseconds > 0:
		time.sleep(milliseconds / 1000.0)

def disposeAll():
	for hamster in Hamster._Hamster__hamsters:
		hamster._Hamster__release()
	Hamster._Hamster__hamsters = []

def __handleSignal(signal, frame):
	disposeAll()
	raise SystemExit

signal.signal(signal.SIGINT, __handleSignal)