# author: Kwang-Hyun Park (akaii@kw.ac.kr)

from roboid import Roboid
from hamster import Hamster
from device import Device
from device import DeviceType
from device import DataType

class HamsterRoboid(Roboid):
	def __init__(self):
		Roboid.__init__(self, 'Hamster', 0x00400000)
		self.__light = 0
		self.__temperature = 0
		self.__wheelBalance = 0
		self.__lineTracerFlag = 0
		self.__lineTracerStateLeft = -1
		self.__lineTracerStateRight = -1

	def getId(self):
		return Hamster.ID

	def __createModel(self):
		from hamster import Hamster
		self.__leftWheelDevice = self._Roboid__addDevice(DeviceType.EFFECTOR, Hamster.LEFT_WHEEL, 'LeftWheel', DataType.INTEGER, 1, 0, -100, 100)
		self.__rightWheelDevice = self._Roboid__addDevice(DeviceType.EFFECTOR, Hamster.RIGHT_WHEEL, 'RightWheel', DataType.INTEGER, 1, 0, -100, 100)
		self.__buzzerDevice = self._Roboid__addDevice(DeviceType.EFFECTOR, Hamster.BUZZER, 'Buzzer', DataType.FLOAT, 1, 0.0, 0, 167772.15)
		self.__outputADevice = self._Roboid__addDevice(DeviceType.EFFECTOR, Hamster.OUTPUT_A, 'OutputA', DataType.INTEGER, 1, 0, 0, 255)
		self.__outputBDevice = self._Roboid__addDevice(DeviceType.EFFECTOR, Hamster.OUTPUT_B, 'OutputB', DataType.INTEGER, 1, 0, 0, 255)
		self.__topologyDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.TOPOLOGY, 'Topology', DataType.INTEGER, 1, 0, 0, 15)
		self.__leftLedDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.LEFT_LED, 'LeftLed', DataType.INTEGER, 1, 0, 0, 7)
		self.__rightLedDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.RIGHT_LED, 'RightLed', DataType.INTEGER, 1, 0, 0, 7)
		self.__noteDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.NOTE, 'Note', DataType.INTEGER, 1, 0, 0, 88)
		self.__lineTracerModeDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.LINE_TRACER_MODE, 'LineTracerMode', DataType.INTEGER, 1, 0, 0, 15)
		self.__lineTracerSpeedDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.LINE_TRACER_SPEED, 'LineTracerSpeed', DataType.INTEGER, 1, 4, 0, 7)
		self.__ioModeADevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.IO_MODE_A, 'IoModeA', DataType.INTEGER, 1, 0, 0, 15)
		self.__ioModeBDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.IO_MODE_B, 'IoModeB', DataType.INTEGER, 1, 0, 0, 15)
		self.__configProximityDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.CONFIG_PROXIMITY, 'ConfigProximity', DataType.INTEGER, 1, 2, 1, 7)
		self.__configGravityDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.CONFIG_GRAVITY, 'ConfigGravity', DataType.INTEGER, 1, 0, 0, 3)
		self.__configBandWidthDevice = self._Roboid__addDevice(DeviceType.COMMAND, Hamster.CONFIG_BAND_WIDTH, 'ConfigBandWidth', DataType.INTEGER, 1, 3, 1, 8)
		self.__signalStrengthDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.SIGNAL_STRENGTH, 'SignalStrength', DataType.INTEGER, 1, 0, -128, 0)
		self.__leftProximityDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.LEFT_PROXIMITY, 'LeftProximity', DataType.INTEGER, 1, 0, 0, 255)
		self.__rightProximityDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.RIGHT_PROXIMITY, 'RightProximity', DataType.INTEGER, 1, 0, 0, 255)
		self.__leftFloorDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.LEFT_FLOOR, 'LeftFloor', DataType.INTEGER, 1, 0, 0, 255)
		self.__rightFloorDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.RIGHT_FLOOR, 'RightFloor', DataType.INTEGER, 1, 0, 0, 255)
		self.__accelerationDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.ACCELERATION, 'Acceleration', DataType.INTEGER, 3, 0, -32768, 32767)
		self.__lightDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.LIGHT, 'Light', DataType.INTEGER, 1, 0, 0, 65535)
		self.__temperatureDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.TEMPERATURE, 'Temperature', DataType.INTEGER, 1, 0, -40, 88)
		self.__inputADevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.INPUT_A, 'InputA', DataType.INTEGER, 1, 0, 0, 255)
		self.__inputBDevice = self._Roboid__addDevice(DeviceType.SENSOR, Hamster.INPUT_B, 'InputB', DataType.INTEGER, 1, 0, 0, 255)
		self.__lineTracerStateDevice = self._Roboid__addDevice(DeviceType.EVENT, Hamster.LINE_TRACER_STATE, 'LineTracerState', DataType.INTEGER, 2, 0, 0, 7)

	def __toHex(self, number):
		value = int(number)
		if value < 0: value += 0x100
		value = hex(value).upper()[2:]
		if len(value) > 1:
			return value
		return '0' + value

	def __toHex3(self, number):
		value = int(number)
		if value < 0: value += 0x1000000
		value = hex(value).upper()[2:]
		length = len(value)
		if length < 6:
			result = ''
			for i in range(length, 6):
				result += '0'
			return result + value
		return value

	def __encode(self, address):
		result = ''
		result += self.__toHex(self.__topologyDevice.read() & 0x0f)
		result += '0010'
		result += self.__toHex(self.__leftWheelDevice.read())
		result += self.__toHex(self.__rightWheelDevice.read())
		result += self.__toHex(self.__leftLedDevice.read())
		result += self.__toHex(self.__rightLedDevice.read())
		result += self.__toHex3(self.__buzzerDevice.read() * 100)
		result += self.__toHex(self.__noteDevice.read())
		mode = self.__lineTracerModeDevice.read()
		if self.__lineTracerModeDevice._Device__isWritten():
			if mode != 0:
				self.__lineTracerFlag ^= 0x80
		temp = (mode & 0x0f) << 3
		temp |= self.__lineTracerSpeedDevice.read() & 0x07
		temp |= self.__lineTracerFlag
		result += self.__toHex(temp)
		result += self.__toHex(self.__configProximityDevice.read())
		temp = (self.__configGravityDevice.read() & 0x0f) << 4
		temp |= self.__configBandWidthDevice.read() & 0x0f
		result += self.__toHex(temp)
		temp = (self.__ioModeADevice.read() & 0x0f) << 4
		temp |= self.__ioModeBDevice.read() & 0x0f
		result += self.__toHex(temp)
		result += self.__toHex(self.__outputADevice.read())
		result += self.__toHex(self.__outputBDevice.read())
		result += self.__toHex(self.__wheelBalance)
		result += '0000-'
		result += address
		result += '\r'
		self._Roboid__clearWritten()
		return result

	def __decode(self, packet):
		packet = str(packet)
		value = int(packet[6:8], 16)
		value -= 0x100
		self.__signalStrengthDevice._Device__put(value)
		value = int(packet[8:10], 16)
		self.__leftProximityDevice._Device__put(value)
		value = int(packet[10:12], 16)
		self.__rightProximityDevice._Device__put(value)
		value = int(packet[12:14], 16)
		self.__leftFloorDevice._Device__put(value)
		value = int(packet[14:16], 16)
		self.__rightFloorDevice._Device__put(value)
		value = int(packet[16:20], 16)
		if value > 0x7fff: value -= 0x10000
		self.__accelerationDevice._Device__putAt(0, value)
		value = int(packet[20:24], 16)
		if value > 0x7fff: value -= 0x10000
		self.__accelerationDevice._Device__putAt(1, value)
		value = int(packet[24:28], 16)
		if value > 0x7fff: value -= 0x10000
		self.__accelerationDevice._Device__putAt(2, value)
		value = int(packet[28:30], 16)
		if value == 0:
			self.__light = int(packet[30:34], 16)
		else:
			value = int(packet[30:32], 16)
			if value > 0x7f: value -= 0x100
			self.__temperature = int(value / 2.0 + 24)
		self.__lightDevice._Device__put(self.__light)
		self.__temperatureDevice._Device__put(self.__temperature)
		value = int(packet[34:36], 16)
		self.__inputADevice._Device__put(value)
		value = int(packet[36:38], 16)
		self.__inputBDevice._Device__put(value)
		value = int(packet[38:40], 16)
		if (value & 0x40) != 0:
			left = (value & 0x38) >> 3
			right = value & 0x07
			if left != self.__lineTracerStateLeft:
				self.__lineTracerStateLeft = left
				self.__lineTracerStateDevice._Device__putAt(0, left)
			if right != self.__lineTracerStateRight:
				self.__lineTracerStateRight = right
				self.__lineTracerStateDevice._Device__putAt(1, right)