# author: Kwang-Hyun Park (akaii@kw.ac.kr)

from named_element import NamedElement
from device import Device
from device import DeviceType

class Roboid(NamedElement):
	def __init__(self, name, mask):
		self.__mask = mask
		self.__roboids = []
		self.__devices = []
		self.__sensoryDevices = []
		self.__motoringDevices = []
		NamedElement.__init__(self, name)

	def __addDevice(self, deviceType, uid, name, dataType, dataSize, initialValue, minValue, maxValue):
		dev = Device(uid, name, deviceType, dataType, dataSize, initialValue, minValue, maxValue)
		self.__devices.append(dev)
		if deviceType == DeviceType.SENSOR or deviceType == DeviceType.EVENT:
			self.__sensoryDevices.append(dev)
		else:
			self.__motoringDevices.append(dev)
		return dev

	def __addRoboid(self, roboid):
		if isinstance(roboid, Roboid):
			self.__roboids.append(roboid)

	def findRoboidByName(self, name):
		name = str(name).lower()
		dot = name.find('.')
		if dot < 0:
			for roboid in self.__roboids:
				if name == roboid.getName().lower():
					return roboid
		else:
			roboidName = name[:dot]
			subName = name[dot+1:]
			for roboid in self.__roboids:
				if roboidName == roboid.getName().lower():
					return roboid.findRoboidByName(subName)
		return None

	def findDeviceByName(self, name):
		name = str(name).lower()
		dot = name.find('.')
		if dot < 0:
			for device in self.__devices:
				if name == device.getName().lower():
					return device
		else:
			roboidName = name[:dot]
			subName = name[dot+1:]
			for roboid in self.__roboids:
				if roboidName == roboid.getName().lower():
					return roboid.findDeviceByName(subName)
		return None

	def findDeviceById(self, deviceId):
		mask = deviceId & 0xfffff000
		if mask == self.__mask:
			for device in self.__devices:
				if device._Device__getUid() == deviceId:
					return device
			return None
		for roboid in self.__roboids:
			device = roboid.findDeviceById(deviceId)
			if device != None:
				return device
		return None

	def addDeviceDataChangedListener(self, listener):
		if listener:
			for device in self.__devices:
				device.addDeviceDataChangedListener(listener)
			for roboid in self.__roboids:
				roboid.addDeviceDataChangedListener(listener)

	def removeDeviceDataChangedListener(self, listener):
		if listener:
			for device in self.__devices:
				device.removeDeviceDataChangedListener(listener)
			for roboid in self.__roboids:
				roboid.removeDeviceDataChangedListener(listener)

	def clearDeviceDataChangedListeners(self):
		for device in self.__devices:
			device.clearDeviceDataChangedListeners()
		for roboid in self.__roboids:
			roboid.clearDeviceDataChangedListeners()

	def __updateSensoryDeviceState(self):
		for device in self.__sensoryDevices:
			device._Device__updateDeviceState()
		for roboid in self.__roboids:
			roboid.__updateSensoryDeviceState()

	def __updateMotoringDeviceState(self):
		for device in self.__motoringDevices:
			device._Device__updateDeviceState()
		for roboid in self.__roboids:
			roboid.__updateMotoringDeviceState()

	def __notifySensoryDeviceDataChanged(self):
		for device in self.__sensoryDevices:
			device._Device__notifyDeviceDataChanged()
		for roboid in self.__roboids:
			roboid.__notifySensoryDeviceDataChanged()

	def __notifyMotoringDeviceDataChanged(self):
		for device in self.__motoringDevices:
			device._Device__notifyDeviceDataChanged()
		for roboid in self.__roboids:
			roboid.__notifyMotoringDeviceDataChanged()

	def __reset(self):
		for device in self.__devices:
			device._Device__reset()
		for roboid in self.__roboids:
			roboid.__reset()

	def __clearWritten(self):
		for device in self.__devices:
			device._Device__clearWritten()