# author: Kwang-Hyun Park (akaii@kw.ac.kr)

from named_element import NamedElement
from roboid import Roboid
from device import Device

class Robot(NamedElement):
	def __init__(self, id, name, index, dispose):
		self.__id = id
		self.__index = index
		self.__roboids = []
		self.__execute = None
		self.__dispose = dispose
		NamedElement.__init__(self, name)

	def getId(self):
		return self.__id

	def getIndex(self):
		return self.__index

	def dispose(self):
		self.__dispose()

	def __addRoboid(self, roboid):
		if isinstance(roboid, Roboid):
			self.__roboids.append(roboid)

	def findRoboidByName(self, name):
		name = str(name).lower()
		dot = name.find('.')
		if dot < 0:
			for roboid in self.__roboids:
				if name == roboid.getName().lower():
					return roboid
		else:
			roboidName = name[:dot]
			subName = name[dot+1:]
			for roboid in self.__roboids:
				if roboidName == roboid.getName().lower():
					return roboid.findRoboidByName(subName)
		return None

	def findDeviceByName(self, name):
		name = str(name).lower()
		dot = name.find('.')
		if dot >= 0:
			roboidName = name[:dot]
			subName = name[dot+1:]
			for roboid in self.__roboids:
				if roboidName == roboid.getName().lower():
					return roboid.findDeviceByName(subName)
		return None

	def findDeviceById(self, deviceId):
		for roboid in self.__roboids:
			device = roboid.findDeviceById(deviceId)
			if isinstance(device, Device):
				return device
		return None

	def e(self, deviceId):
		device = self.findDeviceById(deviceId)
		if device == None: return False
		return device.e()

	def read(self, deviceId, arg=None):
		device = self.findDeviceById(deviceId)
		if device == None: return 0
		return device.read(arg)

	def write(self, deviceId, arg1, arg2=None):
		device = self.findDeviceById(deviceId)
		if device == None: return False
		return device.write(arg1, arg2)

	def addDeviceDataChangedListener(self, listener):
		if listener:
			for roboid in self.__roboids:
				roboid.addDeviceDataChangedListener(listener)

	def removeDeviceDataChangedListener(self, listener):
		if listener:
			for roboid in self.__roboids:
				roboid.removeDeviceDataChangedListener(listener)

	def clearDeviceDataChangedListeners(self):
		for roboid in self.__roboids:
			roboid.clearDeviceDataChangedListeners()

	def setExecutable(self, execute):
		self.__execute = execute

	def __updateSensoryDeviceState(self):
		for roboid in self.__roboids:
			roboid._Roboid__updateSensoryDeviceState()

	def __updateMotoringDeviceState(self):
		for roboid in self.__roboids:
			roboid._Roboid__updateMotoringDeviceState()

	def __notifySensoryDeviceDataChanged(self):
		for roboid in self.__roboids:
			roboid._Roboid__notifySensoryDeviceDataChanged()

	def __notifyMotoringDeviceDataChanged(self):
		for roboid in self.__roboids:
			roboid._Roboid__notifyMotoringDeviceDataChanged()

	def reset(self):
		for roboid in self.__roboids:
			roboid._Roboid__reset()